#!/usr/bin/perl -w
binmode (STDOUT,":utf8");

# Program parser2html.pl

# This program accepts output from AIS_parser.pl and creates an HTML rendition

# Usage options --
# 1) Create intermediate file with parser output...

#   ./AIS_parser.pl -sf input.txt >tempfile
#   ./parser2html.pl <tempfile >output.html

# 2) Pipe parser output to html program...

#   ./AIS_parser.pl -sf input.txt | ./parser2html.pl >output.html

# (c) Gary C. Kessler, 2019

$version = "1.5";
$build_date = "05/11/2019";
$max_out = 100;

print "<html>\n<head>\n<title>AIS Parser</title>\n</head>\n";
print "<!-- Generated by parser2html.pl (v$version, $build_date), (c) Gary Kessler -->\n";
print "<body>\n<blockquote>\n";

# Manage the top of the output page

print "<h3 align=center>\n";
do
  {
  $line = <stdin>;
  chomp $line;
  print "$line<br>\n";
  }
  until (substr ($line,0,5) eq "*****");

print "</h3>\n";

# Process lines with data but before the actual message content parsing (body = 0)

$body = 0;

do
  {
  $line = <stdin>;
  chomp $line;
  if (substr ($line,0,3) ne "---")
      { 
      if ($line eq "")
        { $line = "&nbsp"; }
      if (substr ($line,0,1) eq " ")
        { substr ($line,0,1) = "&nbsp"; }

# Special test for the AIV sentence, which could contain a < or >

      if (substr ($line,0,3) eq "AIV" && ($line =~ m/</ || $line =~ m/>/)) { $line = process_angle ($line); }

      $len = length ($line);
      for ($i=0; $i<$len; $i += $max_out)
        { printf "%s<br>\n", substr ($line,$i,$max_out); }
      }
    else
      {
      $body = 1;
      print "<br>\n<table border=1 cellpadding=5>\n<tr>\n";
      print "<tr><td colspan = 2>\n";
      print "<center>\n";
      print "$line<br>\n";
      print "</center>\n";
      print "</td></tr>\n";
      }
  }
  until $body == 1;

# Now process the content of the message itself.
# Lines that begin with "  **" are a sub-block of information

$block = 0;

do
  {
  $line = <stdin>;
  chomp $line;
  @fields = split ("=", $line);

  HTML:
    {
    scalar (@fields) == 1 && do
      {
      if ($block == 0)
        {
        $block = 1;
        print "<tr><td colspan = 2>\n";
        }

      if (substr ($line,5,4) eq "http")
        {
        $url = substr ($line,5,length($line)-4);
        $line = '&nbsp;&nbsp;** <a href="' . $url . '" target="_blank">' . $url . "</a>";
        }

      if (substr ($line,0,4) eq "  **")
        { $line = '&nbsp;&nbsp;**' . substr ($line,4,length($line)-4); }

      if (substr ($line,0,3) eq "---")
        { print "<center>\n"; }

      print "$line<br>\n";

      if (substr ($line,0,3) eq "---")
        { print "</center>\n"; }

      if (substr ($line,0,3) eq "---" || substr ($line,0,3) eq "###")
        {
        $block = 0;
        print "</td></tr>\n";
        }
      last HTML;
      };

    scalar (@fields) == 2 && do
      {
      if ($block == 1)
        {
        $block = 0;
        print "</td></tr>\n";
        }

# Handle degree symbol

      do
        {
        $chr = index ($fields [1], "\xB0");
        if ($chr >= 0)
          { $fields [1] = substr ($fields [1],0,$chr-1) . "&#176;" . substr ($fields [1], $chr+1, length ($fields [1]) - $chr+1); }
         }
         until ($chr < 0);

       printf "<tr><td>%s</td>\n<td>%s</td></tr>\n", $fields [0], $fields [1];
      last HTML;
      };
    }
  }
  until (substr ($line,0,7) eq "--- End");


print "</table>\n</blockquote>\n</body>\n</html>\n";

# ********************************************************
# process_angle
# ********************************************************

# Handle the situation where an angle bracket (i.e., "<" or ">" appears in an input line.
# These two characters need to be replaced with "&lt;" and "&gt;", respectively.

# We only get here if we know that the input string contains an < or >...

sub process_angle
  {
  my ($string) = @_;
  my ($n,$len);

  do
    {
    $n = index ($string,"<");
    if ($n >= 0)
      { 
      $len = length ($string);
      $string = substr ($string,0,$n) . "&lt;" . substr ($string,$n+1,$len-$n-1);
      }
    }
  until ($n < 0);

  do
    {
    $n = index ($string,">");
    if ($n >= 0)
      {
      $len = length ($string);
      $string = substr ($string,0,$n) . "&gt;" . substr ($string,$n+1,$len-$n-1);
      }
    }
  until ($n < 0);

  return $string;
  }

